package cn.com.qmth.examcloud.task.service;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Date;

import javax.transaction.Transactional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.task.base.ScheduleJob;
import cn.com.qmth.examcloud.task.base.TaskTracker;
import cn.com.qmth.examcloud.task.dao.TaskTraceRepo;
import cn.com.qmth.examcloud.task.dao.entity.TaskTraceEntity;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年7月25日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Component
public class DefaultTaskTracker implements TaskTracker {

	@Autowired
	TaskTraceRepo taskTraceRepo;

	@Transactional
	@Override
	public void start(ScheduleJob scheduleJob, String traceId) {
		TaskTraceEntity entity = new TaskTraceEntity();
		entity.setJobId(scheduleJob.getJobId());
		entity.setStartTime(new Date());
		entity.setTraceId(traceId);
		entity.setJobStatus("启动");
		taskTraceRepo.save(entity);
	}

	@Transactional
	@Override
	public void whenException(ScheduleJob scheduleJob, String traceId, Exception e) {
		TaskTraceEntity entity = taskTraceRepo.findByTraceId(traceId);
		entity.setJobStatus("异常");
		entity.setEndTime(new Date());

		if (e instanceof StatusException) {
			entity.setException(((StatusException) e).toJson());
		} else {
			entity.setException(getStackTrace(e));
		}

		taskTraceRepo.save(entity);
	}

	private String getStackTrace(Throwable throwable) {
		Writer result = new StringWriter();
		PrintWriter printWriter = new PrintWriter(result);
		throwable.printStackTrace(printWriter);
		return result.toString();
	}

	@Transactional
	@Override
	public void onEnd(ScheduleJob scheduleJob, String traceId) {
		TaskTraceEntity entity = taskTraceRepo.findByTraceId(traceId);
		entity.setJobStatus("成功");
		entity.setEndTime(new Date());
		taskTraceRepo.save(entity);

	}

}
