package cn.com.qmth.examcloud.task.service.impl;

import java.lang.reflect.Method;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.api.HandleSyncCloudService;
import cn.com.qmth.examcloud.commons.api.request.SyncBaseRequest;
import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.JsonUtil;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.base.util.RegExpUtil;
import cn.com.qmth.examcloud.task.dao.DataSyncRepo;
import cn.com.qmth.examcloud.task.dao.entity.DataSyncEntity;
import cn.com.qmth.examcloud.task.service.DataSyncService;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年9月19日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class DataSyncServiceImpl implements DataSyncService {

	@Autowired
	DataSyncRepo dataSyncRepo;

	@Autowired
	HandleSyncCloudService handleSyncCloudService;

	@Override
	public boolean sync(String methodName, SyncBaseRequest req, Boolean retry) {
		String group = PropertiesUtil.getString("$sync." + methodName + ".group");
		if (StringUtils.isBlank(group)) {
			throw new StatusException("T-001001", "group is not configured");
		}

		List<String> componentList = RegExpUtil.findAll(group, "[^\\,]+");

		if (CollectionUtils.isEmpty(componentList)) {
			throw new StatusException("T-001001", "no component");
		}

		boolean successful = true;
		for (String component : componentList) {
			boolean result = sync(component, methodName, req, retry);
			if (!result) {
				successful = false;
			}
		}

		return successful;
	}

	@Override
	public boolean sync(String component, String methodName, SyncBaseRequest req, Boolean retry) {
		String url = PropertiesUtil.getString("$sync." + methodName + ".component." + component);
		req.setUrl(url);

		boolean successful = false;
		try {
			Method method = handleSyncCloudService.getClass().getMethod(methodName, req.getClass());
			method.invoke(handleSyncCloudService, req);
			successful = true;
		} catch (Exception e) {
			if (retry) {
				DataSyncEntity entity = new DataSyncEntity();
				entity.setComponent(component);
				entity.setMethodName(methodName);
				entity.setParamType(req.getClass().getName());
				entity.setSyncNum(0);
				entity.setParamJson(JsonUtil.toJson(req));
				dataSyncRepo.saveAndFlush(entity);
			}
		}

		return successful;
	}

}
