package cn.com.qmth.examcloud.task.service.job;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import cn.com.qmth.examcloud.commons.api.request.SyncBaseRequest;
import cn.com.qmth.examcloud.commons.base.util.JsonUtil;
import cn.com.qmth.examcloud.task.base.AbstractTask;
import cn.com.qmth.examcloud.task.base.ScheduleJob;
import cn.com.qmth.examcloud.task.base.TaskTracker;
import cn.com.qmth.examcloud.task.dao.DataSyncRepo;
import cn.com.qmth.examcloud.task.dao.entity.DataSyncEntity;
import cn.com.qmth.examcloud.task.service.DataSyncService;

/**
 * 数据同步任务
 *
 * @author WANGWEI
 * @date 2018年8月2日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Component("dataSyncTask")
public class DataSyncTask extends AbstractTask {

	@Autowired
	TaskTracker TaskTracker;

	@Autowired
	DataSyncRepo dataSyncRepo;

	@Autowired
	DataSyncService dataSyncService;

	@Override
	public TaskTracker getTaskTracker() {
		return TaskTracker;
	}

	@Override
	public void run(ScheduleJob scheduleJob) throws Exception {

		String component = scheduleJob.getExt1();
		String methodName = scheduleJob.getExt1();

		while (true) {
			List<DataSyncEntity> list = dataSyncRepo
					.findTop10ByComponentAndMethodNameOrderById(component, methodName);

			if (CollectionUtils.isEmpty(list)) {
				break;
			}

			for (DataSyncEntity cur : list) {
				String paramType = cur.getParamType();
				Class<?> c = Class.forName(paramType);
				Object obj = JsonUtil.fromJson(cur.getParamJson(), c);
				SyncBaseRequest req = (SyncBaseRequest) obj;
				dataSyncService.sync(component, methodName, req, false);

				dataSyncRepo.delete(cur);
			}

		}

	}

}
