package cn.com.qmth.examcloud.task.starter.config;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

import cn.com.qmth.examcloud.commons.base.util.PathUtil;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.web.interceptor.FirstInterceptor;
import cn.com.qmth.examcloud.commons.web.redis.RedisClient;
import cn.com.qmth.examcloud.commons.web.security.RequestPermissionInterceptor;
import cn.com.qmth.examcloud.commons.web.security.SpringCloudInterceptor;
import cn.com.qmth.examcloud.commons.web.security.bean.Role;
import cn.com.qmth.examcloud.commons.web.security.bean.User;

/**
 * 默认WebMvcConfigurer
 *
 * @author WANGWEI
 * @date 2018年5月22日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Configuration
public class DefaultWebMvcConfigurerAdapter extends WebMvcConfigurerAdapter {

	@Autowired
	RedisClient redisClient;

	static {
		PropertiesUtil.configureAndWatch(PathUtil.getResoucePath("security-mapping.properties"));
	}

	@Override
	public void addInterceptors(InterceptorRegistry registry) {
		registry.addInterceptor(new FirstInterceptor()).addPathPatterns("/**");

		SpringCloudInterceptor springCloudInterceptor = new SpringCloudInterceptor();
		springCloudInterceptor.setRedisClient(redisClient);
		registry.addInterceptor(springCloudInterceptor).addPathPatterns("/**");

		RequestPermissionInterceptor requestPermissionInterceptor = getRequestPermissionInterceptor();
		requestPermissionInterceptor.configureAndWatch("security-exclusions.conf");
		registry.addInterceptor(requestPermissionInterceptor).addPathPatterns("/**");

		super.addInterceptors(registry);
	}

	@Bean
	public RequestPermissionInterceptor getRequestPermissionInterceptor() {
		return new RequestPermissionInterceptor(redisClient) {

			@Override
			public boolean hasPermission(String mappingPath, User user) {
				List<Role> roleList = user.getRoleList();
				if (CollectionUtils.isEmpty(roleList)) {
					return false;
				}

				String roles = PropertiesUtil.getString(mappingPath);
				if (StringUtils.isBlank(roles)) {
					return true;
				}

				roles = "," + roles + ",";

				for (Role role : roleList) {
					if (roles.contains("," + role.getRoleCode() + ",")) {
						return true;
					}
				}

				return false;
			}

		};
	}
}
